package com.cnlaunch.downloadbin;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.net.ProtocolException;
import java.net.URL;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Handler;
import android.util.Log;

import com.cnlaunch.downloadbin.UpdateLogic.OnRequestFinishListener;
import com.cnlaunch.golo.logger.GoloLog;
import com.lidroid.xutils.http.ResponseInfo;

/**
 * 下载守护APK或DownLoadBin
 * 
 *
 */
public class DownloadApk {
	
	// 查询版本服务器地址
	private static String QueryVersion_URL = "https://exchange.huijiacn.com/exchange/access/device/querymodule?";
		
	private static String logInfo = "升级Download模块_DownloadApk_";
	public static ResponseInfos info = null;
	static Context mContext;
	
	
	/**
	 * 查询服务器最新的版本信息
	 * 守护APK和DownloadBin
	 * @param moduleName download : DownLoadBin guardapk :守护apk
	 * @return ResponseInfo
	 * @throws IOException
	 */
	public static void querySoftVersionInfo(final String moduleName,Context mcContext,String sn,final Handler handler) throws IOException{
		mContext = mcContext;
		Log.e("golo", logInfo+"querySoftVersionInfo: "+"【查询服务端apk和download.bin版本信息】");
		UpdateLogic.getInstance().querySoftVersionInfo(new RequestInfo(QueryVersion_URL,sn,1).toQueryUrlString(), new OnRequestFinishListener() {
			
			@Override
			public void onFinish(int code, Object object) {
				@SuppressWarnings("unchecked")
				ResponseInfo<String> responseInfo = (ResponseInfo<String>) object;
				if(code == 0){//成功
					info = new ResponseInfos(responseInfo.result);
					GoloLog.d(logInfo+"querySoftVersionInfo: "+"result:"+responseInfo.result.toString());
					if (info.isResponseInfoValid() == false) {
						Log.e("golo", logInfo+"querySoftVersionInfo: "+"查询版本信息失败");
						handler.sendEmptyMessage(1);
						return;
					}
					if(moduleName != null && moduleName.equals("download")){//升级DownLoadBin
						updateDownLoadBin(info,handler);
					}
				} else if (code == -1){//失败
					handler.sendEmptyMessage(-1);
					return;
				}
			}
		});
	}
	/**
	 * 更新DownLoadBin
	 * @param responseInfo
	 */
	public static void updateDownLoadBin(ResponseInfos responseInfo, Handler handler){
		if (DownLoadThread.isDownloadBinNeedUpdate(responseInfo, "DOWNLOAD.BIN")) {
			boolean flag = false;
			try {
				flag = downloadModule(responseInfo.getDownloadBinUrl(),mContext, "DOWNLOAD.BIN", responseInfo.downloadBinVersion);
				if (flag) {
					handler.sendEmptyMessage(4);
					Log.e("golo", logInfo+"downloadDownloadBin: "+"下载DOWNLOAD.BIN成功");
				} else {
					handler.sendEmptyMessage(5);
					Log.e("golo", logInfo+"downloadDownloadBin: "+"下载DOWNLOAD.BIN失败");
				}
			} catch (IOException e) {
				Log.e("golo", logInfo+"downloadDownloadBin: "+"下载异常", e);
				handler.sendEmptyMessage(3);
			}
		}else {
			Log.e("golo", logInfo+"downloadDownloadBin: "+"DOWNLOAD.BIN无需更新");
			handler.sendEmptyMessage(2);
		}
	}
	
	
	private static boolean isApkNeedUpdate(ResponseInfos responseInfo, String apkName, String apkPackageName) {			
		PackageInfo apkInfo = null;		
		int apkVersionCode = 0;
		String apkVersionName = "00.00";
		int responseVersionCode = 0;
		try {
			apkInfo = mContext.getPackageManager().getPackageInfo(apkPackageName, 0);
			apkVersionCode = apkInfo.versionCode;
			apkVersionName = apkInfo.versionName;
		} catch (NameNotFoundException e) {
			Log.e("golo", logInfo+"isApkNeedUpdate: "+"获取包信息异常", e);
			e.printStackTrace();
		}
		try {
			if (apkInfo != null && apkVersionCode != 0) {
				if (apkName.equals("rcuGuardService.apk")) {
					Log.e("golo", logInfo+"isApkNeedUpdate: "+"开始对比守护APK版本");
					responseVersionCode = (int)(Float.parseFloat(responseInfo.rcuApkVersion)*100);
					if (!responseInfo.rcuApkVersion.equals(apkVersionName) && apkVersionCode < responseVersionCode) {
						return true;
					}
				}
			} else {
				Log.e("golo", logInfo+"isApkNeedUpdate: "+"为读取到"+apkName+"版本信息，强制升级");
				return true;
			}
		} catch (NumberFormatException e) {
			Log.e("golo", logInfo+"isApkNeedUpdate: "+"查询版本返回的参数有问题", e);
		}
		return false;
	}
			
	/**
	 * 下载APK文件
	 * @param softurl 下载地址
	 * @param context 上下文对象
	 * @param moduleFileName 下载文件名称
	 * @param moduleVersionName 下载版本名称
	 * @throws IOException
	 */
	public static boolean downloadModule(String softurl, Context context, String moduleFileName, String moduleVersionName) throws IOException{
		Log.e("golo", logInfo+"downloadModule: "+"开始下载："+moduleFileName);
		URL url = new URL(softurl);
		HttpURLConnection connection = null;
		try {
			connection = connetToServer(url);
		} catch (IOException e) {
			Log.e("golo", logInfo+"downloadModule: "+"连接http异常", e);
		}
		String filePath = DownLoadThread.DOWNLOADBIN_PATH + moduleFileName;
		File file = new File(filePath);
		if (file.exists()) {
			file.delete();
		}
		if (connection.getResponseCode() ==200) {
			//download.bin 要特殊处理
			readInputStreamToFile(connection, filePath,moduleFileName,"V"+moduleVersionName);
			setDownloadedFlag(context, moduleFileName, true);
			return true;
		}else{
			setDownloadedFlag(context, moduleFileName, false);
			return false;
		}
	}

		
	private static void setDownloadedFlag(Context context, String moduleFileName, boolean flag) {
		mContext = context;
		if (flag) {
			//DOWNLOAD.BIN  已经在下载时进行 保存
			if (!moduleFileName.equals("DOWNLOAD.BIN")) {
				SharedPreferences conferPreferencesReady = context.getSharedPreferences(moduleFileName, Context.MODE_PRIVATE);
				conferPreferencesReady.edit().putBoolean("finishFlag", true).commit();
			}
		}else {
			if (moduleFileName.equals("DOWNLOAD.BIN")) {
				putdSharedPreferencesData(context, 0, null, null, false);
			} else {
				SharedPreferences conferPreferencesReady = context.getSharedPreferences(moduleFileName, Context.MODE_PRIVATE);
				conferPreferencesReady.edit().putBoolean("finishFlag", false).commit();
			}
		}
	}
	
	private static HttpURLConnection connetToServer(URL url) throws IOException, ProtocolException {
		HttpURLConnection conn2 = (HttpURLConnection) url.openConnection();
		conn2.setConnectTimeout(30 * 1000);
		conn2.setRequestMethod("GET");
		conn2.setRequestProperty("Connection", "Keep-Alive");
		conn2.setRequestProperty("Charset", "UTF-8");
		conn2.setRequestProperty("content-type", "application/vnd.android.package-archive");
		conn2.setDoInput(true);
		conn2.setDoOutput(true);
		return conn2;
	}


	private static int readInputStreamToFile(HttpURLConnection connection, String filePath,String moduleFileName,String moduleVersionName)throws IOException {			
		InputStream in = null;
		OutputStream out = null;
		MessageDigest digest = null;
		try {
			in = connection.getInputStream();
			out = new FileOutputStream(new File(filePath), false);
		} catch (FileNotFoundException e){
			Log.e("golo", logInfo+"readInputStreamToFile: "+"获取输入输出流异常", e);
			e.printStackTrace();
		}
		byte[] data = new byte[4096];
		int fileLen = 0;
		try {
			digest = MessageDigest.getInstance("MD5");
		} catch (NoSuchAlgorithmException e){
			Log.e("golo", logInfo+"readInputStreamToFile: "+"MessageDigest.getInstance异常", e);
			e.printStackTrace();
		}
		while (true) {
			int len = 0;
			if (in != null) {
				len = in.read(data, 0, data.length);
			}
			if (len <= 0) {
				if (in != null) {
					in.close();
				}
				if (out != null) {
					out.close();
				}
				break;
			} else {
				Log.e("golo", logInfo+"readInputStreamToFile: "+moduleFileName+"【正在下载】：" + fileLen);
				if (out != null) {
					out.write(data, 0, len);
					digest.update(data, 0, len);
					fileLen += len;
				}
			}
		}
		// 保存md5值
		BigInteger bigInt = new BigInteger(1, digest.digest());
		Log.e("golo", logInfo+"readInputStreamToFile: "+moduleFileName+"【md5值：】" + bigInt.toString(16));
		Log.e("golo", logInfo+"readInputStreamToFile: "+moduleFileName+"【文件长度：】" + fileLen);
		if (moduleFileName.equals("DOWNLOAD.BIN")) {
			putdSharedPreferencesData(mContext, fileLen, bigInt.toString(16), moduleVersionName, true);
		}
		return fileLen;
	}

	/**
	 * 保存数据
	 * @Team Golo Terminal
	 * @date: 2014-6-10
	 * @param context
	 * @param totalLen
	 * @param MD5
	 * @param version
	 * @param flag
	 * @return void
	 */
	public static void putdSharedPreferencesData(Context context, int totalLen, String MD5, String version, boolean flag) {
		SharedPreferences downloadInfoPreferences = context.getSharedPreferences("downloadInfo", Context.MODE_PRIVATE);
		if (flag){
			downloadInfoPreferences.edit().putBoolean("downloadBINFlag", flag).commit();
			downloadInfoPreferences.edit().putInt("totalLen", totalLen).commit();
			downloadInfoPreferences.edit().putString("version", version).commit();
			downloadInfoPreferences.edit().putString("MD5", MD5).commit();
		} else{
			downloadInfoPreferences.edit().putBoolean("downloadBINFlag", flag).commit();
			downloadInfoPreferences.edit().putInt("totalLen", 0).commit();
			downloadInfoPreferences.edit().putString("version", null).commit();
			downloadInfoPreferences.edit().putString("MD5", null).commit();
		}
	}
	
}
