package com.cnlaunch.downloadbin;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Build;
import android.util.AttributeSet;
import android.widget.Button;

/**
 * @className: LcButton (LaunchButton缩写)
 * @discription: 自定义button，替换xml文件中传入string
 * @author: zenghaigeng
 * @time: 2015年04月07日 下午13:40:13
 */
@SuppressLint("NewApi")
public class LcButton extends Button {

    public LcButton(Context context) {
        super(context);
    }

    public LcButton(Context context, AttributeSet attrs) {
        super(context, attrs);
        setDefaultText(context, attrs);
    }

    public LcButton(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        setDefaultText(context, attrs);
    }

    /**
     * @title: setDefaultText
     * @discription: 设置默认文本
     * @param: context 容器
     * @param: attrs 属性集合
     */
    private void setDefaultText(Context context, AttributeSet attrs) {
        if (LcResources.isResourceNotReady())
            return;
        for (int i = 0; i < attrs.getAttributeCount(); i++) {
            if (attrs.getAttributeName(i).equals("text")) {
                try {
                    setText(LcResources.getLcResourceText(context, attrs, i), BufferType.NORMAL);
                } catch (Exception e) {
                    //e.printStackTrace();
                }
            } else if(attrs.getAttributeName(i).equals("textColor")) {
                setTextViewColor(attrs.getAttributeResourceValue(i, 0));
            } else if(attrs.getAttributeName(i).equals("background")) {
                setButtonBackground(attrs.getAttributeResourceValue(i, 0));
            }
        }
    }

    /**
     * @title: setTextViewColor
     * @discription: 设置字体颜色
     */
    private void setTextViewColor(int resId) {
        try {
            setTextColor(LcResources.getInstance().getColorStateList(resId));
        } catch (Exception e) {
            //e.printStackTrace();
        }
    }

    /**
     * @title: setButtonBackground
     * @discription: 设置字体颜色
     */
    private void setButtonBackground(int resId) {
        try {
            if(LcResources.getInstance().getLcResourceTypeName(getResources(), resId).equals("color")) {
                setBackgroundColor(LcResources.getInstance().getColor(resId));
            } else {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                    setBackground(LcResources.getInstance().getDrawable(resId));
                } else {
                    setBackgroundDrawable(LcResources.getInstance().getDrawable(resId));
                }
            }
        } catch (Exception e) {
            //e.printStackTrace();
        }
    }
}
