package com.cnlaunch.downloadbin;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import android.content.Context;
import android.content.res.AssetManager;
import android.content.res.ColorStateList;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.content.res.TypedArray;
import android.graphics.drawable.Drawable;
import android.os.Environment;
import android.util.AttributeSet;
import android.util.DisplayMetrics;

import com.cnlaunch.golo.logger.GoloLog;

/**
 * @className: LcResources (LaunchResources缩写)
 * @discription: 自定义资源文件类，允许替换系统资源文件
 * @author: zenghaigeng
 * @time: 2015年04月07日 下午13:40:13
 */
public class LcResources extends Resources {
    public static final String TAG = LcResources.class.getCanonicalName();
    /**
     * 资源文件根目录
     */
    private static final String RESOURCE_PATH = Environment.getExternalStorageDirectory() + "/cnlaunch/golo3/resource/";
    /**
     * 从资源文件来的id，默认为首个string的id
     */
    public static final int ID_FROM_SDCARD = 0x7f020000;
    /**
     * LcResources对象
     */
    private static LcResources instance;
    /**
     * sd卡资源文件的resources对象
     */
    private static Resources rs;
    /**
     * 客户端是否需要重启（切换语言用到）
     */
    public static boolean isRestart;
    /**
     * 客户端是否需要重置资源包（下载完资源包用到）
     */
    public static boolean isReset;

    /**
     * 资源类型
     */
    public enum ResourceType {
        /**
         * 字符串 *
         */
        string,
        /**
         * 数组 *
         */
        array,
        /**
         * 图片 *
         */
        drawable,
        /**
         * 颜色 *
         */
        color
    }

    public LcResources(AssetManager assets, DisplayMetrics metrics, Configuration config) {
        super(assets, metrics, config);
    }

    @Override
    public CharSequence getText(int id) {
        if (rs == null) {
            return super.getText(id);
        } else {
            try {
                return rs.getText(getLcResourceId(id, ResourceType.string.name()));
            } catch (Exception e) {
                //e.printStackTrace();
                return super.getText(id);
            }
        }
    }

    @Override
    public CharSequence getText(int id, CharSequence def) {
        if (rs == null) {
            return super.getText(id, def);
        } else {
            try {
                return rs.getText(getLcResourceId(id, ResourceType.string.name()), def);
            } catch (Exception e) {
                //e.printStackTrace();
                return super.getText(id, def);
            }
        }
    }

    @Override
    public String getString(int id, Object... formatArgs) {
        if (rs == null) {
            return super.getString(id, formatArgs);
        } else {
            try {
                return rs.getString(getLcResourceId(id, ResourceType.string.name()), formatArgs);
            } catch (Exception e) {
                //e.printStackTrace();
                return super.getString(id, formatArgs);
            }
        }
    }

    @Override
    public TypedArray obtainTypedArray(int id) throws NotFoundException {
        if (rs == null) {
            return super.obtainTypedArray(id);
        } else {
            try {
                return rs.obtainTypedArray(getLcResourceId(id, ResourceType.array.name()));
            } catch (Exception e) {
                //e.printStackTrace();
                return super.obtainTypedArray(id);
            }
        }
    }

    @Override
    public String[] getStringArray(int id){
        if (rs == null) {
            return super.getStringArray(id);
        } else {
            try {
                TypedArray strs = super.obtainTypedArray(id);
                List<String> list = new ArrayList<String>();
                for (int i = 0; i < strs.length(); i++) {
                    list.add(getString(strs.getResourceId(i, 0)));
                }
                strs.recycle();
                return list.toArray(new String[list.size()]);
            } catch (Exception e) {
                //e.printStackTrace();
                return super.getStringArray(id);
            }
        }
    }

    @Override
    public Drawable getDrawable(int id) throws NotFoundException {
        if (rs == null) {
            return super.getDrawable(id);
        } else {
            try {
                return rs.getDrawable(getLcResourceId(id, ResourceType.drawable.name()));
            } catch (Exception e) {
                //e.printStackTrace();
                return super.getDrawable(id);
            }
        }
    }

    @Override
    public ColorStateList getColorStateList(int id) throws NotFoundException {
        if (rs == null) {
            return super.getColorStateList(id);
        } else {
            try {
                return rs.getColorStateList(getLcResourceId(id, ResourceType.color.name()));
            } catch (Exception e) {
                //e.printStackTrace();
                return super.getColorStateList(id);
            }
        }
    }

    @Override
    public int getColor(int id) throws NotFoundException {
        if (rs == null) {
            return super.getColor(id);
        } else {
            try {
                return rs.getColor(getLcResourceId(id, ResourceType.color.name()));
            } catch (Exception e) {
                //e.printStackTrace();
                return super.getColor(id);
            }
        }
    }

    /*@Override
    Drawable loadDrawable(TypedValue value, int id)
            throws NotFoundException {
        Resources resource = context.getResources();
        Class clazz = resource.getClass();
        try {
            Method method = clazz.getDeclaredMethod("loadDrawable");
            return (Drawable)method.invoke(value, id);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }*/

    /**
     * @title: getLcResource
     * @discription: 获取Resource对象实例
     */
    public static Resources getLcResource() {
        return rs;
    }

    /**
     * @title: getInstance
     * @discription: 获取资源对象实例
     */
    public static LcResources getInstance() {
        return instance;
    }

    /**
     * @param resources
     * @title: setInstance
     * @discription: 设置资源对象实例
     */
    public static void setInstance(LcResources resources) {
        instance = resources;
    }

    /**
     * @title: isResourceNotReady
     * @discription: 是否不允许加载动态资源包
     */
    public static boolean isResourceNotReady() {
        return rs == null;
    }

    /**
     * @title: reset
     * @discription: 重置资源文件
     */
    public static void reset() {
        instance = null;
        rs = null;
    }

    /**
     * @title: getLcResourceTypeName
     * @discription: 获取资源类型
     */
    public static String getLcResourceTypeName(Resources resource, int resId) {
        return resource.getResourceTypeName(resId);
    }

    /**
     * @title: getLcResourceId
     * @discription: 获取资源包资源id
     * @param: id 默认工程的资源id
     * @param: type id类型
     */
    public int getLcResourceId(int id, String type) throws Exception {
        return rs.getIdentifier(getResourceEntryName(id), type, rs.getResourcePackageName(ID_FROM_SDCARD));
    }

    /**
     * @title: getLcResourceText
     * @discription: 获取资源包资源字符串
     * @param: attrs 属性集合
     * @param: index 位置索引
     * @return: string 资源字符串
     */
    public static String getLcResourceText(Context context, AttributeSet attrs, int index) throws Exception {
        int id = attrs.getAttributeResourceValue(index, 0);
        int resourceId = rs.getIdentifier(context.getResources().getResourceEntryName(id), ResourceType.string.name(), rs.getResourcePackageName(ID_FROM_SDCARD));
        return rs.getString(resourceId);
    }

    /**
     * @title: switchLanguage
     * @discription: 修改默认语言
     * @param locale
     */
    public static void switchLanguage(Locale locale) {
        if(rs == null) {
            GoloLog.e("no resource pack founded");
            return;
        }
        Configuration config = rs.getConfiguration();
        DisplayMetrics dm = rs.getDisplayMetrics();
        config.locale = locale;
        rs.updateConfiguration(config, dm);
    }
}
