package com.cnlaunch.downloadbin;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Build;
import android.util.AttributeSet;
import android.widget.TextView;

/**
 * @className: LcTextView (LaunchTextView缩写)
 * @discription: 自定义TextView，替换xml文件中传入string
 * @author: zenghaigeng
 * @time: 2015年04月07日 下午13:40:13
 */
@SuppressLint("NewApi")
public class LcTextView extends TextView {

    public LcTextView(Context context) {
        super(context);
    }

    public LcTextView(Context context, AttributeSet attrs) {
        super(context, attrs);
        setDefaultValue(context, attrs);
    }

    public LcTextView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        setDefaultValue(context, attrs);
    }

    /**
     * @title: setDefaultValue
     * @discription: 设置xml默认属性
     * @param: context 容器
     * @param: attrs 属性集合
     */
    private void setDefaultValue(Context context, AttributeSet attrs) {
        if (LcResources.isResourceNotReady())
            return;
        for (int i = 0; i < attrs.getAttributeCount(); i++) {
            if (attrs.getAttributeName(i).equals("text")) {
                try {
                    setText(LcResources.getLcResourceText(context, attrs, i), BufferType.NORMAL);
                } catch (Exception e) {
                    //e.printStackTrace();
                }
            } else if(attrs.getAttributeName(i).equals("drawableLeft")) {
                setTextViewCompoundDrawables(attrs.getAttributeResourceValue(i, 0), 0, 0, 0);
            } else if(attrs.getAttributeName(i).equals("drawableTop")) {
                setTextViewCompoundDrawables(0, attrs.getAttributeResourceValue(i, 0), 0, 0);
            } else if(attrs.getAttributeName(i).equals("drawableRight")) {
                setTextViewCompoundDrawables(0, 0, attrs.getAttributeResourceValue(i, 0), 0);
            } else if(attrs.getAttributeName(i).equals("drawableBottom")) {
                setTextViewCompoundDrawables(0, 0, 0, attrs.getAttributeResourceValue(i, 0));
            } else if(attrs.getAttributeName(i).equals("textColor")) {
                setTextViewColor(attrs.getAttributeResourceValue(i, 0));
            } else if(attrs.getAttributeName(i).equals("background")) {
                setTextViewBackground(attrs.getAttributeResourceValue(i, 0));
            }
        }
    }

    /**
     * @title: setTextViewCompoundDrawables
     * @discription: 根据id设置图片drawable
     */
    private void setTextViewCompoundDrawables(int left, int top, int right, int bottom) {
        try {
            setCompoundDrawablesWithIntrinsicBounds(left != 0 ? LcResources.getInstance().getDrawable(left) : null, top != 0 ? LcResources.getInstance().getDrawable(top) : null, right != 0 ? LcResources.getInstance().getDrawable(right) : null, bottom != 0 ? LcResources.getInstance().getDrawable(bottom) : null);
        } catch (Exception e) {
            //e.printStackTrace();
        }
    }

    /**
     * @title: setTextViewColor
     * @discription: 设置字体颜色
     */
    private void setTextViewColor(int resId) {
        try {
            setTextColor(LcResources.getInstance().getColorStateList(resId));
        } catch (Exception e) {
            //e.printStackTrace();
        }
    }

    /**
     * @title: setTextViewBackground
     * @discription: 设置字体背景颜色
     */
    private void setTextViewBackground(int resId) {
        try {
            if(LcResources.getInstance().getLcResourceTypeName(getResources(), resId).equals("color")) {
                setBackgroundColor(LcResources.getInstance().getColor(resId));
            } else {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                    setBackground(LcResources.getInstance().getDrawable(resId));
                } else {
                    setBackgroundDrawable(LcResources.getInstance().getDrawable(resId));
                }
            }
        } catch (Exception e) {
            //e.printStackTrace();
        }
    }

    @Override
    public void setCompoundDrawablesWithIntrinsicBounds(int left, int top, int right, int bottom) {
        super.setCompoundDrawablesWithIntrinsicBounds(left, top, right, bottom);
        try {
            setCompoundDrawablesWithIntrinsicBounds(left != 0 ? LcResources.getLcResource().getDrawable(left) : null, top != 0 ? LcResources.getLcResource().getDrawable(top) : null, right != 0 ? LcResources.getLcResource().getDrawable(right) : null, bottom != 0 ? LcResources.getLcResource().getDrawable(bottom) : null);
        } catch (Exception e) {
            //e.printStackTrace();
        }
    }
}
