package com.cnlaunch.downloadbin;

import java.io.File;

import android.app.Activity;
import android.app.Dialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnKeyListener;
import android.content.Intent;
import android.content.SharedPreferences;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.view.WindowManager;
import android.view.WindowManager.LayoutParams;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.cnlaunch.golo.inspection.LoginActivity;
import com.cnlaunch.golo.inspection.tools.StringUtils;
import com.cnlaunch.golo.inspection.view.GoloProgressDialog;
import com.cnlaunch.golo.wifisettings2.R;

/**
 * @version 1.0
 * @author: ljm
 * @类说明: downloadBIN升级进度广播接受
 * @创建时间：2014-11-25 下午7:57:46
 */
public class UpgradeDownloadBINReceiver extends BroadcastReceiver implements OnClickListener {

    public static final String DOWNLOADBIN_UPGRADE_ERROR = "com.golo3.action.DownloadBin_Upgrade_Error";
    public static final String DOWNLOADBIN_UPGRADE_PROGRESS = "com.golo3.action.DownloadBin_Upgrade_Progress";
    public static final String DOWNLOADBIN_UPGRADE_COMPLETE = "com.golo3.action.DownloadBin_Upgrade_Complete";
    public static final String DOWNLOADBIN_DPU_VERSION = "com.golo3.action.DPU_Version";
       
    public static boolean isRegister;
    private String serialNo;

    private Dialog dialog;
    private TextView tvDialogTitle;
    private ProgressBar mProgressBar;
    private TextView progressText;
    private TextView tvError;
    private Button btnDialogCancel;

    public UpgradeDownloadBINReceiver(String serialNo) {
        this.serialNo = serialNo;
    }

    /**
     * @return void    返回类型
     * @throws
     * @Title: initDownDialog
     * @说明:初始化下载对话框
     * @参数:
     */
    private void initDownDialog(Context context) {
        if (dialog == null) {
            dialog = new Dialog(context);
            dialog.setOnKeyListener(new OnKeyListener() {
                @Override
                public boolean onKey(DialogInterface dialog, int keyCode, KeyEvent event) {
                    if (keyCode == KeyEvent.KEYCODE_BACK && event.getRepeatCount() == 0) {
                        return true;
                    } else {
                        return false;
                    }
                }
            });
            dialog.setCanceledOnTouchOutside(false);
            dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
            View v = LayoutInflater.from(context).inflate(R.layout.daig_download_car_config_dialog_layout, null);
            tvDialogTitle = (TextView) v.findViewById(R.id.tv_dialog_title);
            mProgressBar = (ProgressBar) v.findViewById(R.id.pb_dialog_download_progress);
            tvError = (TextView) v.findViewById(R.id.tv_dialog_error);
            progressText = (TextView) v.findViewById(R.id.tv_dialog_download_progress);
            btnDialogCancel = (Button) v.findViewById(R.id.btn_dialog_cancel);
            tvDialogTitle.setText(R.string.upgrade_downloadbin_file);
            tvError.setText(R.string.upgrade_downloadbin_failed);
            btnDialogCancel.setVisibility(View.GONE);
            /************luojunqing   2015/12/17   start 给取消按钮设置设置监听点击可取消**********/
            btnDialogCancel.setOnClickListener(this);
            /************luojunqing   2015/12/17   end 给取消按钮设置设置监听点击可取消**********/
            dialog.setContentView(v);
            dialog.getWindow().setBackgroundDrawableResource(android.R.color.transparent);

            int width = getScreenWidthAndHeight(context)[0];
            WindowManager.LayoutParams params = dialog.getWindow().getAttributes();
            params.width = width / 10 * 9;
            params.height = LayoutParams.WRAP_CONTENT;
            dialog.getWindow().setAttributes(params);
            progressText.setText("0%");
            mProgressBar.setProgress(0);

            if (!((Activity) context).isFinishing() && !dialog.isShowing()) {
                dialog.show();
            }
        }
    }
    
	/**
	 * 获得屏幕的大小
	 * 
	 * @return new int[]{width,height}
	 */
	public static int[] getScreenWidthAndHeight(Context context) {
		DisplayMetrics dm = new DisplayMetrics();
		((Activity) context).getWindowManager().getDefaultDisplay().getMetrics(dm);
		return new int[] { dm.widthPixels, dm.heightPixels };
	}


    @Override
    public void onReceive(Context context, Intent intent) {
        String action = intent.getAction();
        if (dialog == null && !(action.equals(DOWNLOADBIN_DPU_VERSION))) {
            initDownDialog(context);
        }
        if (action.equals(DOWNLOADBIN_UPGRADE_ERROR)) {
            GoloProgressDialog.dismissProgressDialog(context);
            tvError.setVisibility(View.VISIBLE);
            progressText.setVisibility(View.GONE);
            mProgressBar.setVisibility(View.GONE);
            btnDialogCancel.setVisibility(View.VISIBLE); 
            if (isRegister) {
                context.unregisterReceiver(this); //取消注册DOWNLOAD_BIN升级广播
            }
        } else if (action.equals(DOWNLOADBIN_UPGRADE_PROGRESS)) {
            GoloProgressDialog.dismissProgressDialog(context);
            int progress = intent.getIntExtra("downloadbin_upgrade_progress", 0);
            mProgressBar.setProgress(progress);
            progressText.setText(progress + "%");     
        } else if (action.equals(DOWNLOADBIN_UPGRADE_COMPLETE)) {
            mProgressBar.setProgress(100);
            progressText.setText("100%");
            if (dialog != null && dialog.isShowing()) {
                dialog.dismiss();
            }
            
            if (isRegister) {
                context.unregisterReceiver(this); //取消注册DOWNLOAD_BIN升级广播
            }
            
        	String filePath = DownLoadThread.DOWNLOADBIN_PATH + "DOWNLOAD.BIN";
    		File file = new File(filePath);
    		if (file.exists()) {
    			file.delete();
    		}
            //不提交信息给服务端
//            ConnectorTool.uploadDownloadBINVersion(new ConnectorManagerInterface(context), serialNo, versionNo);
//            GoloActivityManager.create().finishActivity((Activity) context);
        }else if(action.equals(DOWNLOADBIN_DPU_VERSION)){
        	String device_serialno = "";
        	String dpu_version= "V0.01";
    		if(null!=intent){
    			if(intent.hasExtra("device_serialno")){
    				 device_serialno= intent.getStringExtra("device_serialno");
    			}  			
    			if(intent.hasExtra("dpu_version")){
    				 dpu_version= intent.getStringExtra("dpu_version");
    				 if(StringUtils.isEmpty(dpu_version) || dpu_version == null){
    					 dpu_version= "V0.01";
    				 }
    			}
    		}
    		
    		if (isRegister) {
                context.unregisterReceiver(this); //取消注册DOWNLOAD_BIN升级广播
           }
    		
    		 if(StringUtils.isEmpty(device_serialno) || device_serialno == null){
    			 Log.e("golo", "用户自动取消连接蓝牙"); 
    			 GoloProgressDialog.dismissProgressDialog(context);
				 return;
			 }
    		
			SharedPreferences downloadInfoPreferences = context.getSharedPreferences("downloadInfo", Context.MODE_PRIVATE);
			downloadInfoPreferences.edit().putString(device_serialno, dpu_version).commit();
			Log.e("golo", "获取接头downloadbin版本: device_serialno = " + device_serialno +" dpu_version = " + dpu_version);

			//获取到接头downloadbin版本开始查询是否需要更新
			Thread thread = new Thread(new DownLoadThread(context,LoginActivity.mHandler,device_serialno));
			thread.start();
        }
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btn_dialog_cancel:
                if (dialog != null && dialog.isShowing()) {
                    dialog.dismiss();
                }
                break;
            default:
                break;
        }
    }
}
