package com.cnlaunch.golo.inspection;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.text.InputFilter;
import android.text.InputType;
import android.text.SpannableStringBuilder;
import android.text.method.NumberKeyListener;
import android.text.style.ForegroundColorSpan;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.cnlaunch.golo.inspection.tools.AllCapTransformationMethod;
import com.cnlaunch.golo.tools.GoloActivityManager;
import com.cnlaunch.golo.wifisettings2.R;

/**
 * 
 * @ClassName: ModifyVehicleVinCodeActivity 
 * @说明:Vin码输入界面
 * @author ljm
 * @date 2014-12-13 下午1:26:05
 */
public class ModifyVehicleVinCodeActivity extends BaseActivity implements OnClickListener {

	private EditText etVin;
	private TextView vinExplain;

	private String vinCode;

	public static final char[] vinCodeValidChars = new char[] { 
		'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 
		'd', 'e', 'f', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p',
		'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C',
		'D', 'E', 'F', 'G', 'H', 'J', 'K', 'L', 'M', 'N', 'P',
		'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z' };

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		initView(R.string.car_vin, R.layout.activity_modify_vehicle_vin, R.drawable.titlebar_sure_icon);

		Intent intent = getIntent();
		if (intent.hasExtra("vinCode")) {
			vinCode = intent.getStringExtra("vinCode");
		}

		initUI();
	}

	private void initUI() {
		etVin = (EditText) findViewById(R.id.editVinCode);
		vinExplain = (TextView) findViewById(R.id.vinExplain);

		etVin.setTransformationMethod(new AllCapTransformationMethod());
		etVin.setKeyListener(new NumberKeyListener() {//实现字符串过滤，限定可输入的字符类型
			@Override
			public int getInputType() {
				return InputType.TYPE_MASK_VARIATION;  
			}

			@Override
			protected char[] getAcceptedChars() {
				return vinCodeValidChars;
			}
		});

		etVin.setFilters(new InputFilter[]{new InputFilter.LengthFilter(17)}); 

		if (vinCode != null) {
			etVin.setText(vinCode);
		}

		String simUseRemind = vinExplain.getText().toString();
		SpannableStringBuilder stringBuilder = new SpannableStringBuilder(simUseRemind);
		ForegroundColorSpan greenColorSpan = new ForegroundColorSpan(getResources().getColor(R.color.green_text_color));
		ForegroundColorSpan grayColorSpan = new ForegroundColorSpan(getResources().getColor(R.color.gray_text_color));
		stringBuilder.setSpan(greenColorSpan, 0, 3, SpannableStringBuilder.SPAN_EXCLUSIVE_EXCLUSIVE);
		stringBuilder.setSpan(grayColorSpan, 3, simUseRemind.length(), SpannableStringBuilder.SPAN_INCLUSIVE_INCLUSIVE);
		vinExplain.setText(stringBuilder);
	}

	@Override
	protected void rightClick(int index) {
		//		Utils.hiddenSoftKeyboard(this, etVin);
		vinCode = etVin.getText().toString().trim();
		((InputMethodManager) getSystemService(INPUT_METHOD_SERVICE)).hideSoftInputFromWindow(getCurrentFocus().
				getWindowToken(),InputMethodManager.HIDE_NOT_ALWAYS);
		if (!vinCode.equals("")) {
			if ((vinCode.equals("")||(vinCode.length()==17))) {
				vinCode = vinCode.toUpperCase();
				Intent data = new Intent();
				data.putExtra("vinCode", vinCode);
				setResult(Activity.RESULT_OK, data);
				GoloActivityManager.create().finishActivity(this);
			}else{
				Toast.makeText(ModifyVehicleVinCodeActivity.this, "请输入正确的VIN码！", Toast.LENGTH_SHORT).show();
			}
		}else{
			Toast.makeText(ModifyVehicleVinCodeActivity.this, "VIN码不能为空", Toast.LENGTH_SHORT).show();
		}
	}

}
