package com.cnlaunch.golo.inspection.logic;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;

import com.cnlaunch.golo.GoloApplication;
import com.cnlaunch.golo.inspection.model.CarCord;
import com.cnlaunch.golo.inspection.model.CarModel;
import com.cnlaunch.golo.inspection.model.CarSeries;
import com.cnlaunch.golo.inspection.model.ConfigBean;
import com.cnlaunch.golo.inspection.model.JSONMsg;
import com.cnlaunch.golo.inspection.tools.SharePreferenceUtils;
import com.cnlaunch.golo.inspection.tools.SignUtils;
import com.cnlaunch.golo.inspection.tools.StringUtils;
import com.cnlaunch.golo.logger.GoloLog;
import com.cnlaunch.golo.tools.CommonUtils;
import com.lidroid.xutils.HttpUtils;
import com.lidroid.xutils.exception.HttpException;
import com.lidroid.xutils.http.ResponseInfo;
import com.lidroid.xutils.http.callback.RequestCallBack;
import com.lidroid.xutils.http.client.HttpRequest.HttpMethod;

public class RequestLogic {
	
	private HttpUtils http;
	private static RequestLogic logic;
	private RequestLogic(){
		http=new HttpUtils();
	}
	public synchronized static RequestLogic getInstance(){
		if(logic==null){
			logic=new RequestLogic();
		}
		return logic;
	}
	
	public  interface OnRequestFinishListener{
		//code=0 请求成功；code=1,失败,tag:标记是哪个请求的回调,一般为方法名
		void onFinish(int code,String tag,Object object);
	}
	/**
	 * 注册接口
	 * @param listener
	 */
	public void login(final OnRequestFinishListener listener){
		Map<String, String> params=new HashMap<String, String>();
		params.put("action", "develop.reg_user");
		String param=SignUtils.getSign(params);
		http.send(HttpMethod.GET, "http://open.api.dbscar.com/?"+param, new RequestCallBack<String>() {

			@Override
			public void onSuccess(ResponseInfo<String> responseInfo) {
				// TODO Auto-generated method stub
				JSONMsg jsonMsg = new JSONMsg();
				JSONObject result = null;
				try {
					// 对JSON数据进行解析
					jsonMsg.decode(new JSONObject(responseInfo.result));
					if(0!=(jsonMsg.getCode())){
						listener.onFinish(1,"login", null);
						return;
					}
					result = jsonMsg.getJsonObject();
					if(result!=null&&result.length()>0) {
						SharePreferenceUtils utils=new SharePreferenceUtils((Context) listener);
						utils.saveAccessData(result.getString("access_id"), result.getString("access_token"));
						GoloApplication.access_id=result.getString("access_id");
						GoloApplication.access_token=result.getString("access_token");
					}
					
					listener.onFinish(0,"login", null);
					
				} catch (Exception e) {
					e.printStackTrace();
				} 
			}

			@Override
			public void onFailure(HttpException error, String msg) {
				// TODO Auto-generated method stub
				GoloLog.e(msg);
				listener.onFinish(1,"login", null);
				} 
		});
	}
	/**
	 * 解除绑定接头
	 * @param serial_no
	 * @param listener
	 */
	public void releaseConnector(String serial_no,final OnRequestFinishListener listener){
		Map<String, String> params = new HashMap<String, String>();
		params.put("action",
				"product_service.cancel_mycar_register");
		params.put("develop_id", GoloApplication.develop_id);
		params.put("time", Long.toString(System
				.currentTimeMillis() / 1000));
		params.put("serial_no", serial_no);
		String sign = SignUtils.getSign(
				GoloApplication.delevop_key, params);
		params.put("sign", sign);
		String param = SignUtils.createLinkString(params);
		http.send(HttpMethod.GET,
				"http://open.api.dbscar.com/?" + param,
				new RequestCallBack<String>() {

					@Override
					public void onSuccess(
							ResponseInfo<String> responseInfo) {
						JSONMsg jsonMsg = new JSONMsg();
						try {
							jsonMsg.decode(new JSONObject(
									responseInfo.result));
							if (0 == jsonMsg.getCode()) {
								listener.onFinish(0,"releaseConnector", null);
							} else {
								listener.onFinish(1,"releaseConnector", null);
							}
						} catch (JSONException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
							listener.onFinish(1,"releaseConnector", null);
						}
					}

					@Override
					public void onFailure(HttpException error,
							String msg) {
						listener.onFinish(1,"releaseConnector", null);
					}
				});
	}
	/**
	 * 激活接头接口
	 * @param serial_no
	 * @param password
	 * @param carcord
	 * @param listener
	 */
	public void activateConnector(String serial_no,String password,CarCord carcord,final OnRequestFinishListener listener){
		Map<String, String> params=new HashMap<String, String>();
		params.put("action", "forjth.save_car_config");
		params.put("car_carcase_num", carcord.getCar_brand_vin());//vin码
		params.put("mine_car_plate_num", carcord.getMine_car_plate_num());//车牌号
		if(!StringUtils.isEmpty(carcord.getCar_sub_type_id())){
			params.put("code_id", carcord.getCar_sub_type_id());//车辆品牌ID
		}else{
			params.put("code_id", carcord.getCar_series_id());//车辆品牌ID
		}
		params.put("car_type_id", carcord.getCar_type_id());//车型
		params.put("car_producing_year", carcord.getCar_producing_year());//年款
		params.put("car_displacement",carcord.getCar_displacement());//排量
		params.put("car_gearbox_type",carcord.getCar_gearbox_type());//档，  0手动 1自动 默认自动
		params.put("serial_no",serial_no);//序列号
		params.put("password",password);//密码
		params.put("display_lan", "cn");
		String param=SignUtils.getSign(params);
		param = param.replaceAll(" ", "%20");
		http.send(HttpMethod.GET, "http://open.api.dbscar.com/?"+param,  new RequestCallBack<String>() {

			@Override
			public void onSuccess(ResponseInfo<String> responseInfo) {
				// TODO Auto-generated method stub
				try {
					JSONObject jsonObject=new JSONObject(responseInfo.result);
					if(0==jsonObject.getInt("code")){
//						SharePreferenceUtils utils=new SharePreferenceUtils((Context) listener);
//						String car_configId = "",obd_config="",user_id="",car_sign="",obd_sign="";
//						if(jsonObject.has("car_configId")){
//							car_configId=jsonObject.getString("car_configId");
//						}
//						if(jsonObject.has("obd_config")){
//							car_configId=jsonObject.getString("obd_config");
//						}
//						if(jsonObject.has("user_id")){
//							car_configId=jsonObject.getString("user_id");
//						}
//						if(jsonObject.has("car_sign")){
//							car_configId=jsonObject.getString("car_sign");
//						}
//						if(jsonObject.has("obd_sign")){
//							car_configId=jsonObject.getString("obd_sign");
//						}
//						utils.saveConnectorActivateInfo(car_configId,obd_config,user_id,car_sign,obd_sign);
//						GoloLog.e(car_configId+","+obd_config+","+user_id+","+car_sign+","+obd_sign);
//						Toast.makeText((Context)listener, car_configId+","+obd_config+","+user_id+","+car_sign+","+obd_sign, Toast.LENGTH_SHORT).show();
						listener.onFinish(0, "activateConnector", null);
					}else{
						listener.onFinish(jsonObject.getInt("code"), "activateConnector", null);
					}
				} catch (JSONException e) {
					e.printStackTrace();
				}
			}

			@Override
			public void onFailure(HttpException error, String msg) {
				// TODO Auto-generated method stub
				GoloLog.e(msg);
				listener.onFinish(1, "activateConnector", null);
			}
		});
	}
	
	/**
	 * 获取车型接口
	 * @param detailId
	 * @param listener
	 */
	public void getVehicleModels(String detailId,final OnRequestFinishListener listener){
		//http://apps.api.dbscar.com/?action=mine_car_service.query_market_car_type&lan_id_or_name=cn&detailId=%@
				http.send(HttpMethod.GET, "http://apps.api.dbscar.com/?action=mine_car_service.query_market_car_type"
						+ "&lan_id_or_name=cn&detailId="+detailId, new RequestCallBack<String>() {

							@Override
							public void onSuccess(ResponseInfo<String> responseInfo) {
								JSONMsg jsonMsg = new JSONMsg();
								List<CarModel> list = null;
								try {
									//对JSON数据进行解析
										jsonMsg.decode(new JSONObject(responseInfo.result));
										JSONArray array = jsonMsg.getJsonArray();
										if(array != null){
											list = new ArrayList<CarModel>();
											for (int i = 0; i < array.length(); i++) {
												JSONObject jsonobject = array.getJSONObject(i);
												CarModel model = new CarModel();
												if(jsonobject.has("carType") && !CommonUtils.isEmpty(jsonobject.getString("carType")))
													model.setCarType(jsonobject.getString("carType"));
												if(jsonobject.has("id") && !CommonUtils.isEmpty(jsonobject.getString("id")))
													model.setId(jsonobject.getString("id"));
												if(jsonobject.has("diagCarModel") && !CommonUtils.isEmpty(jsonobject.getString("diagCarModel")))
													model.setDiagCarModel(jsonobject.getString("diagCarModel"));
												if(jsonobject.has("detailName") && !CommonUtils.isEmpty(jsonobject.getString("detailName")))
													model.setDetailName(jsonobject.getString("detailName"));
												if(jsonobject.has("lanId") && !CommonUtils.isEmpty(jsonobject.getString("lanId")))
													model.setLanId(jsonobject.getString("lanId"));
												if(jsonobject.has("detailId") && !CommonUtils.isEmpty(jsonobject.getString("detailId")))
													model.setDetailId(jsonobject.getString("detailId"));
												list.add(model);
											}
										}
										listener.onFinish(0, "getVehicleModels", list);
								} catch (JSONException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
									listener.onFinish(1, "getVehicleModels", null);
								}finally{
								}
							}

							@Override
							public void onFailure(HttpException error, String msg) {
								GoloLog.e(msg);
								listener.onFinish(1, "getVehicleModels", null);
							}
						});
	}
	
	
	/**
	 * 获取车辆品牌接口
	 * @param listener
	 */
	 public void getVehicleBrand(final OnRequestFinishListener listener){
		 Map<String, String> params=new HashMap<String, String>();
			params.put("action", "forjth.getbrandid");
			String param=SignUtils.getSign(params);
			//请求车辆品牌列表
			http.send(HttpMethod.GET, "http://open.api.dbscar.com/?"+param, new RequestCallBack<String>() {

				@Override
				public void onSuccess(ResponseInfo<String> responseInfo) {
					List<CarSeries> list = null;
					try {
						//对JSON数据进行解析
						JSONArray array = new JSONArray(responseInfo.result);
						if(array != null){
							list = new ArrayList<CarSeries>();
							for (int i = 0; i < array.length(); i++) {
								JSONObject object = array.getJSONObject(i);
								CarSeries carSeries = new CarSeries();
								if (object.has("autoCode"))
									carSeries.setAutoCode(object.getString("autoCode"));
								carSeries.setCarSeriesId(object.getString("carSeriesId"));
								carSeries.setCarSeriesName(object.getString("carSeriesName"));
								if(object.has("isHot")){
									carSeries.setIsHot(object.getInt("isHot"));
								}
								if(object.has("isAbroad")){
									carSeries.setIsAbroad(object.getInt("isAbroad"));
								}
								if (object.has("subList")
										&& !object.getString("subList").equals("[]")
										&& object.getJSONArray("subList").length() > 0) {
									JSONArray carArray = object.getJSONArray("subList");
									List<CarSeries> subList1 = new ArrayList<CarSeries>();
									for (int j = 0; j < carArray.length(); j++) {
										JSONObject objects = carArray.getJSONObject(j);
										CarSeries carList = new CarSeries();
										carList.setCarSeriesId(objects.getString("carSeriesId"));
										carList.setCarSeriesName(objects
												.getString("carSeriesName"));
										if(object.has("isHot")){
											carSeries.setIsHot(object.getInt("isHot"));
										}
										if(object.has("isAbroad")){
											carSeries.setIsAbroad(object.getInt("isAbroad"));
										}
										if(object.has("autoCode")){
											carList.setAutoCode(objects.getString("autoCode"));
										}
										subList1.add(carList);
									}
									carSeries.setSubList(subList1);
								}
								list.add(carSeries);
							}
							listener.onFinish(0, "getVehicleBrand", list);
						}
					} catch (JSONException e) {
						e.printStackTrace();
						listener.onFinish(1, "getVehicleBrand", null);
					}finally{
					}
				}

				@Override
				public void onFailure(HttpException error, String msg) {
					// TODO Auto-generated method stub
					GoloLog.e(msg);
					listener.onFinish(1, "getVehicleBrand", null);
				}
			});
	 }
	 //获取配置文件下载的参数
	 /**
	  * 获取配置文件下载需要的参数
	  * @param serial_no
	  * @param listener
	  */
	 public void getConfigParams(String serial_no,final OnRequestFinishListener listener){
		 Map<String, String> params=new HashMap<String, String>();
			params.put("action", "data_develop.get_car_eobd");
			params.put("app_id", GoloApplication.app_id);
			params.put("develop_id", GoloApplication.develop_id);
			params.put("serial_no", serial_no);
			params.put("time", Long.toString(System.currentTimeMillis()/1000));
			params.put("sign", SignUtils.getSign(GoloApplication.delevop_key, params));
			String param=SignUtils.createLinkString(params);
			http.send(HttpMethod.GET, "http://open.api.dbscar.com/?"+param, new RequestCallBack<String>() {
				@Override
				public void onSuccess(ResponseInfo<String> responseInfo) {
						JSONMsg jsonMsg=new JSONMsg();
						try {
							jsonMsg.decode(new JSONObject(responseInfo.result));
							if(jsonMsg.getCode()==0){
								JSONObject jsonObject=jsonMsg.getJsonObject();
								SharePreferenceUtils utils=new SharePreferenceUtils((Context) listener);
								String configId = "",obd_config="",user_id="",car_sign="",obd_sign="";
								if(jsonObject.has("configId")){
									configId=jsonObject.getString("configId");
								}
								if(jsonObject.has("obd_config")){
									obd_config=jsonObject.getString("obd_config");
								}
								if(jsonObject.has("user_id")){
									user_id=jsonObject.getString("user_id");
								}
								if(jsonObject.has("sign")){
									car_sign=jsonObject.getString("sign");
								}
								if(jsonObject.has("obd_sign")){
									obd_sign=jsonObject.getString("obd_sign");
								}
								utils.saveConnectorActivateInfo(configId,obd_config,user_id,car_sign,obd_sign);
								ConfigBean bean=new ConfigBean(configId, obd_config, user_id, car_sign, obd_sign);
								listener.onFinish(0, "getConfigParams", bean);
							}else{
								listener.onFinish(jsonMsg.getCode(), "getConfigParams", null);
							}
						} catch (JSONException e) {
							e.printStackTrace();
							listener.onFinish(1, "getConfigParams", null);
						}
				}
				@Override
				public void onFailure(HttpException error, String msg) {
					GoloLog.e(msg);
					listener.onFinish(1, "getConfigParams", null);
				}
			});
	 }
	 /**
	  * 获取序列号对应的车辆信息
	  * @param serial_no
	  * @param listener
	  */
	 public void getCarInfo(String serial_no,final OnRequestFinishListener listener){
		 Map<String, String> params=new HashMap<String, String>();
			params.put("action", "data_develop.get_mine_car_info");
			params.put("app_id", GoloApplication.app_id);
			params.put("develop_id", GoloApplication.develop_id);
			params.put("serial_no", serial_no);
			params.put("time", Long.toString(System.currentTimeMillis()/1000));
			params.put("sign", SignUtils.getSign(GoloApplication.delevop_key, params));
			String param=SignUtils.createLinkString(params);
			http.send(HttpMethod.GET, "http://open.api.dbscar.com/?"+param, new RequestCallBack<String>() {
				@Override
				public void onSuccess(ResponseInfo<String> responseInfo) {
						JSONMsg jsonMsg=new JSONMsg();
						GoloLog.i("responseInfo查询车辆信息=="+responseInfo.result);
						listener.onFinish(0, "getCarInfo", responseInfo.result);
				}
				@Override
				public void onFailure(HttpException error, String msg) {
					GoloLog.e(msg);
				}
			});
	 }
	 
	 /**
	  * 
	  * 加载dpusys.ini文件的内容
	  * @param serial_no
	  * @param listener
	  */
	 public void downloadSysIni(String serial_no,String car_type,final OnRequestFinishListener listener){
		 Map<String, String> params=new HashMap<String, String>();
			params.put("action", "data_develop.get_config_info");
			params.put("app_id", GoloApplication.app_id);
			params.put("develop_id", GoloApplication.develop_id);
			params.put("display_lan", "cn");
			params.put("car_type", car_type);
			params.put("serial_no", serial_no);
			params.put("time",Long.toString(System.currentTimeMillis()/1000));
			params.put("sign", SignUtils.getSign(GoloApplication.delevop_key, params));
			String param=SignUtils.createLinkString(params);
			http.send(HttpMethod.GET, "http://open.api.dbscar.com/?"+param, new RequestCallBack<String>() {

				@Override
				public void onSuccess(ResponseInfo<String> responseInfo) {
					JSONMsg jsonMsg=new JSONMsg();
					try {
						jsonMsg.decode(new JSONObject(responseInfo.result));
						if(jsonMsg.getCode()==0){
							JSONObject jsonObject=jsonMsg.getJsonObject();
							if(0==jsonObject.getInt("code")){
								String result=jsonObject.getString("iniFileContent");
								listener.onFinish(0, "downloadSysIni", result);
							}else{
								listener.onFinish(1, "downloadSysIni", null);
							}
						}else{
							listener.onFinish(1, "downloadSysIni", null);
						}
					} catch (JSONException e) {
						e.printStackTrace();
					}
						
				}

				@Override
				public void onFailure(HttpException error, String msg) {
					// TODO Auto-generated method stub
					GoloLog.e(msg);
					listener.onFinish(1, "downloadSysIni", null);
				}
			});
	 }
	 
}
