package com.cnlaunch.golo.inspection.view;

import java.io.File;
import java.lang.reflect.Method;

import android.content.Context;
import android.content.res.AssetManager;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.os.Environment;
import android.util.AttributeSet;
import android.util.DisplayMetrics;

/**
 * @className: LcResources (LaunchResources缩写)
 * @discription: 自定义资源文件类，允许替换系统资源文件
 * @author: zenghaigeng
 * @time: 2015年04月07日 下午13:40:13
 */
public class LcResources extends Resources {
	public static final String TAG = LcResources.class.getCanonicalName();
	/** 从资源文件来的id，默认为ic_launch的id */
	public static final int ID_FROM_SDCARD = 0x7f020000;
	/** LcResources对象 */
	public static LcResources instance;
	/** sd卡资源文件的resources对象 */
	public static Resources rs;
	/** 资源类型 */
	enum ResourceType {
		/** 字符串 **/
		string,
		/** 数组 **/
		array
	}
	
	public LcResources(Context context, AssetManager assets, DisplayMetrics metrics,
			Configuration config) {
		super(assets, metrics, config);
	}
	
	@Override
	public CharSequence getText(int id) {
		if(rs == null) {
			return super.getText(id);
		} else {
			try {
				return rs.getText(getResourceId(id, ResourceType.string.name()));
			} catch (Exception e) {
				e.printStackTrace();
				return super.getText(id);
			}
		}
	}

	@Override
	public CharSequence getText(int id, CharSequence def) {
		if(rs == null) {
			return super.getText(id, def);
		} else {
			try {
				return rs.getText(getResourceId(id, ResourceType.string.name()), def);
			} catch (Exception e) {
				e.printStackTrace();
				return super.getText(id, def);
			}
		}
	}
	
	@Override
	public CharSequence[] getTextArray(int id) {
		if(rs == null) {
			return super.getTextArray(id);
		} else {
			try {
				return rs.getTextArray(getResourceId(id, ResourceType.array.name()));
			} catch (Exception e) {
				e.printStackTrace();
				return super.getTextArray(id);
			}
		}
	}

	@Override
	public String getString(int id, Object... formatArgs) {
		if(rs == null) {
			return super.getString(id, formatArgs);
		} else {
			try {
				return rs.getString(getResourceId(id, ResourceType.string.name()), formatArgs);
			} catch (Exception e) {
				e.printStackTrace();
				return super.getString(id, formatArgs);
			}
		}
	}

	@Override
	public String[] getStringArray(int id) {
		if(rs == null) {
			return super.getStringArray(id);
		} else {
			try {
				return rs.getStringArray(getResourceId(id, ResourceType.array.name()));
			} catch (Exception e) {
				e.printStackTrace();
				return super.getStringArray(id);
			}
		}
	}

	/**
	 * @title: getResourceId
	 * @discription: 获取资源包资源id
	 * @param: id 默认工程的资源id
	 * @param: type id类型
	 */
	private int getResourceId(int id, String type) throws Exception {
		// like this : getResourceName(id) --> com.cnlaunch.reflecttesta:string/app_name
		return rs.getIdentifier(getResourceName(id).split("/")[1], type, 
				LcResources.rs.getResourcePackageName(ID_FROM_SDCARD));
	}
	
	/**
	 * @title: getResourceText
	 * @discription: 获取资源包资源字符串
	 * @param: attrs 属性集合
	 * @param: index 位置索引
	 * @return: string 资源字符串
	 */
	public static String getResourceText(Context context, AttributeSet attrs, int index) throws Exception {
		int id = attrs.getAttributeResourceValue(index, 0);
		int resourceId = rs.getIdentifier(context.getResources().getResourceName(id).split("/")[1], 
				ResourceType.string.name(), rs.getResourcePackageName(ID_FROM_SDCARD));
		return rs.getString(resourceId);
	}
	
	/**
	 * @title: initResources
	 * @discription: 初始化资源包的resources对象
	 * @param: resource 系统资源类
	 */
	public static void initResources(Resources resource) {
        try {
        	String path = getResourceFilePath();
        	if(path != null) {
        		AssetManager assetManager = AssetManager.class.newInstance();
                Method addAssetPath = assetManager.getClass().getMethod("addAssetPath", String.class);
                addAssetPath.invoke(assetManager, path);
                rs = new Resources(assetManager, resource.getDisplayMetrics(), resource.getConfiguration());
        	}
        } catch (Exception e) {
            e.printStackTrace();
        }
	}
	
	/**
	 * @title: getResourceFilePath
	 * @discription: 获取资源文件路径
	 */
	private static String getResourceFilePath(){ 
        boolean sdCardExist = Environment.getExternalStorageState().equals(android.os.Environment.MEDIA_MOUNTED); 
        if(sdCardExist) { 
        	String path = Environment.getExternalStorageDirectory()+"/"+"resources_zh.ap_";
    		if(new File(path).exists()) {
    			return path;
    		} else {
        		com.cnlaunch.golo.logger.GoloLog.e("no resource file");
        	}
        }   
        return null; 
	 }
}
